/*
 * Copyright (c) 2004, Adam VanderHook <acidos@users.sf.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.bj.pfpro.gui.swing;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import net.bj.pfpro.firewall.Firewall;
import net.bj.pfpro.gui.swing.panel.MacroPanel;
import net.bj.pfpro.gui.swing.panel.NATPanel;
import net.bj.pfpro.gui.swing.panel.RedirectPanel;
import net.bj.pfpro.gui.swing.panel.RulePanel;
import net.bj.pfpro.gui.swing.panel.ScrubPanel;

import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.mapping.MappingException;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;

/**
 * This is the default GUI for PfPro
 * 
 * @author acidos
 * @version $Id: InternalFrame.java,v 1.11.4.1 2004/08/31 00:25:54 acidos Exp $
 */
public class InternalFrame extends JInternalFrame implements ActionListener {

	// Data
	private File file;
	private JFileChooser fc;
	private Firewall firewall;
	
	private JMenuItem mnuFileSave;
	
	public InternalFrame(File newFile) throws Exception {
		// Initial Setup
		super("",	// title bar
			true,	// resizable
			true,	// closable
			true,	// maximizable
			true);	// iconifiable
		setSize(640, 480);
		getContentPane().setLayout(new BorderLayout());
		setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
		
		// File Chooser
		fc = new JFileChooser();
		
		// Handle Document
		if(newFile != null) {
			setFile(newFile);
			
			// Unmarshall document
			FileReader data = new FileReader(getFile());
			Unmarshaller un = new Unmarshaller(Firewall.class);
			firewall = (Firewall)un.unmarshal(data);
			data.close();
		} else {
			firewall = new Firewall();
		}

		// Menu
		JMenuBar mbMain = new JMenuBar();
		
		//// File Menu
		JMenu mnuFile = new JMenu("File");
		mnuFile.setMnemonic(KeyEvent.VK_F);
		mbMain.add(mnuFile);
		
		mnuFile.add(createMenu("New", KeyEvent.VK_N, KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK), "new", (MainFrame)this.getParent()));
		
		JMenuItem mnuFileSave = createMenu("Save", KeyEvent.VK_S, KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK), "save", this);
		
		if(getFile() == null) {
			mnuFileSave.setEnabled(false);
		}
		
		mnuFile.add(mnuFileSave);
		mnuFile.add(createMenu("Save As...", KeyEvent.VK_A, null, "save_as", this));
		mnuFile.add(new JSeparator());
		mnuFile.add(createMenu("Generate", KeyEvent.VK_G, KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK), "transform", this));
		mnuFile.add(new JSeparator());
		mnuFile.add(createMenu("Close", KeyEvent.VK_W, KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK), "close", this));
	
		setJMenuBar(mbMain);

		// Panels
		JPanel panMacros = new MacroPanel(firewall);	// Macros
		JPanel panScrub = new ScrubPanel(firewall);		// Scrub
		JPanel panRules = new RulePanel(firewall);		// Rules
		JPanel panNat = new NATPanel(firewall);			// NAT
		JPanel panRdr = new RedirectPanel(firewall);	// RDR

		// Split the NAT and RDR panel
		JSplitPane panNATRDR = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false, panNat, panRdr);

		// Place Object panel
		JSplitPane panObjects = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false, panMacros, panScrub);
//		getContentPane().add(panObjects, BorderLayout.CENTER);
		
		// The Tab Pane
		JTabbedPane tbPan = new JTabbedPane();
		tbPan.add("Filtering", panRules);
		tbPan.add("NAT", panNATRDR);
		
		// The Split Pane
		JSplitPane panMain = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false, panObjects, tbPan);
		getContentPane().add(panMain, BorderLayout.CENTER);
	}
	
	/**
	 * Generates a new <code>JMenuItem</code>
	 * 
	 * @param label			Label to be displayed
	 * @param mnemonic		Mnemonic to use
	 * @param keyStroke		Accelerator
	 * @param command		Action Command
	 * @param al			Acition Listener
	 * @return JMenuItem
	 */
	private JMenuItem createMenu(String label, int mnemonic, KeyStroke keyStroke, String command, ActionListener al) {
		JMenuItem result = new JMenuItem(label);
		
		result.setMnemonic(mnemonic);
		result.setAccelerator(keyStroke);
		result.setActionCommand(command);
		result.addActionListener(al);
		
		return result;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if("close".equals(e.getActionCommand())) {
			try {
				setClosed(true);
				dispose();
			} catch (PropertyVetoException e1) {
				// Intentionally blank
			}
		} else if("save_as".equals(e.getActionCommand())) {
			File output = getSaveDestination();
			
			if(output != null) {
				saveDocument(output);
				setFile(output);
			}
		} else if("save".equals(e.getActionCommand())) {
			saveDocument(getFile());
		} else if("transform".equals(e.getActionCommand())) {
			TransformerFactory tfactory = TransformerFactory.newInstance();
			File output = getSaveDestination();
			
			if(output != null) {
				try {
					Transformer transformer = tfactory.newTransformer(new StreamSource(getClass().getResourceAsStream("xml2pf.xsl")));
					transformer.setParameter("timestamp", new Date());
					transformer.setParameter("user", System.getProperty("user.name"));
					transformer.transform(new javax.xml.transform.stream.StreamSource(getFile()), new javax.xml.transform.stream.StreamResult(new java.io.FileOutputStream(output)));
				} catch (Exception e1) {
					JOptionPane.showInternalMessageDialog(this, "Unable to perform transform", "Transform Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		} else {
			JOptionPane.showInternalMessageDialog(this, "An unsupported command \"" + e.getActionCommand() + "\" has been requested.  This is a programming error.  Please contact acidos@bandwidth-junkies.net", "Command Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Prompts the user for a file to write to.  If the file exists, the user is prompted whether to overwrite
	 * the file or not.
	 * <p>
	 * In the event that the selected file does not already exist, or that the selected file <i>does</i> exist
	 * and the user has chosen to overwrite it, a <code>java.io.File</code> is returned.  Otherwise, a
	 * <code>null</code>is returned.
	 * 
	 * @return	File	File to save to; <code>null</code> if user changed their mind.
	 */
	private File getSaveDestination() {
		File result = null;
		
		int returnVal = fc.showSaveDialog(this);
		
		// User selected a file
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			int overwrite = JOptionPane.NO_OPTION;
			
			// Overwrite file if it exists?
			if(fc.getSelectedFile().exists()) {
				overwrite = JOptionPane.showConfirmDialog(this, "Overwrite " + fc.getSelectedFile().getName() + "?");	
			} else {
				overwrite = JOptionPane.YES_OPTION;
			}
			
			if(overwrite == JOptionPane.YES_OPTION) {
				result = fc.getSelectedFile();
			}
		}
		
		return result;
	}
	
	/**
	 * Writes the firewall DOM object to the specified file name.
	 * 
	 * @param file	File to write the firewall configuration to
	 */
	protected void saveDocument(File file) {
		try {
			// Marshalling
			FileWriter marshalFile = new FileWriter(file.toString());

			Mapping mapping = new Mapping(getClass().getClassLoader()); 
			mapping.loadMapping(getClass().getResource("mapping.xml"));

			Marshaller m = new Marshaller(marshalFile);
			m.setMapping(mapping);

			m.marshal(firewall);
			marshalFile.close();
		} catch(MappingException mape) {
			// TODO auto-generated catch block
			mape.printStackTrace();
			System.exit(2);
		} catch(MarshalException marshe) {
			// TODO auto-generated catch block
			marshe.printStackTrace();
			System.exit(3);
		} catch(ValidationException vale) {
			// TODO auto-generated catch block
			vale.printStackTrace();
			System.exit(4);
		} catch (IOException e) {
			JOptionPane.showInternalMessageDialog(this, "Unable to save firewall", "File Write Error", JOptionPane.WARNING_MESSAGE);
		}
	}
	
	/**
	 * Retrieves the name of the file that is displayed by the instance
	 * of <code>InternalFrame</code>
	 * 
	 * @return		File associated with data
	 */
	public File getFile() {
		return file;
	}
	
	/**
	 * Sets the file we are working with
	 * 
	 * @param file	File to save data to
	 */
	public void setFile(File file) {
		this.file = file;
		
		if(mnuFileSave != null) {
			mnuFileSave.setEnabled(true);
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see javax.swing.JInternalFrame#getTitle()
	 */
	public String getTitle() {
		String result = "[ Untitled ]";
		
		if(getFile() != null) {
			result = getFile().getName();
		}
		
		return result;
	}

}
