/*
 * Copyright (c) 2004, Adam VanderHook <acidos@users.sf.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.bj.pfpro.gui.swing.panel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableModel;

import net.bj.pfpro.firewall.Firewall;
import net.bj.pfpro.gui.swing.PopupListener;
import net.bj.pfpro.gui.swing.table.VectorCellEditor;
import net.bj.pfpro.gui.swing.table.VectorCellRenderer;

/**
 * A <code>JPanel</b> with a default <code>JPopupMenu</code> and
 * <code>JTable</code> (with a renderer and editor set for
 * <code>Vector</code>'s.
 * <p>
 * In most instances you should <b>extend</b> this class instead of
 * using it directly.
 * <p>
 * <b>NOTE:</b> You must manually call <code>addTable()</code> in
 * the constructor of any child classes in order for the
 * <code>JTable</code> to be displayed.  This is a dirty hack that
 * can potentially be fixed by extending <code>BoxLayout</code>.
 * Please see the documentation for <code>addTable()</code> below
 * for more information.
 * 
 * @author acidos
 * @version $Id: PfPanel.java,v 1.4.4.1 2004/08/27 01:56:03 acidos Exp $
 * @see #addTable()
 * @see java.util.Vector
 * @see javax.swing.JTable
 * @see javax.swing.JPanel
 * @see javax.swing.JPopupMenu
 */
abstract class PfPanel extends JPanel implements ActionListener {

	private Firewall firewall;
	private JTable table;
	private TableModel model;
	private JPopupMenu popupMenu;

	/**
	 * Default constructor.
	 * 
	 * @param firewall	The firewall with data to be displayed
	 * @param model		The TableModel to be used for display
	 */
	protected PfPanel(Firewall firewall, TableModel model) {
		setFirewall(firewall);
		setModel(model);
		
		// Configure the table
		JTable table = new JTable(getModel());
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);	// this enables vertical scrolling
		setTable(table);
		
		// Popup Menu
		setPopupMenu(new JPopupMenu());
		getTable().add(getPopupMenu());
		getTable().addMouseListener(new PopupListener(getPopupMenu()));
		
		// Layout
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		// Handle Vector Editor and Renderer
		getTable().setDefaultRenderer(Vector.class, new VectorCellRenderer());
		getTable().setDefaultEditor(Vector.class, new VectorCellEditor());
	}
	
	/**
	 * Adds the JTable to the Panel.
	 * <p>
	 * Not all child classes of this Panel are alike.  Because of this,
	 * it is often desirable to add additional labels, toolbars, or other
	 * GUI objects <b>before</b> adding the table.  Since we cannot specify
	 * that the table is the <b>last</b> object with any available layout
	 * managers, this is a dirty fix to get around it.
	 */
	protected void addTable() {
		this.add(new JScrollPane(getTable()));
	}

	/**
	 * @return Returns the model.
	 */
	protected TableModel getModel() {
		return model;
	}
	/**
	 * @param model The model to set.
	 */
	protected void setModel(TableModel model) {
		this.model = model;
	}
	/**
	 * @return Returns the table.
	 */
	protected JTable getTable() {
		return table;
	}
	/**
	 * @param table The table to set.
	 */
	protected void setTable(JTable table) {
		this.table = table;
	}
	/**
	 * @param firewall The firewall to set.
	 */
	protected void setFirewall(Firewall firewall) {
		this.firewall = firewall;
	}

	/**
	 * Create a <code>JMenuItem</code> for use internally
	 * with a popup menu.
	 * 
	 * @param title		Label of the menu item
	 * @param action	ActionCommand the menu item calls
	 * @return	JMenuItem
	 */
	protected JMenuItem createMenu(String title, String action) {
		JMenuItem result = new JMenuItem(title);
		
		result.setActionCommand(action);
		result.addActionListener(this);
		
		return result;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public abstract void actionPerformed(ActionEvent e);

	/**
	 * @return Returns the popupMenu.
	 */
	protected JPopupMenu getPopupMenu() {
		return popupMenu;
	}
	/**
	 * @param popupMenu The popupMenu to set.
	 */
	private void setPopupMenu(JPopupMenu popupMenu) {
		this.popupMenu = popupMenu;
	}

}
