/*
 * Copyright (c) 2004, Adam VanderHook <acidos@users.sf.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.bj.pfpro.gui.swing.table;

import java.util.Vector;

import net.bj.pfpro.firewall.Firewall;
import net.bj.pfpro.firewall.Redirect;

/**
 * Handles showing of rules in a JTable.
 * 
 * @author acidos
 * @version $Id: RedirectTableModel.java,v 1.2 2004/08/08 01:28:44 acidos Exp $
 * @see net.bj.pfpro.gui.swing.table.DOMTableModel
 */
public class RedirectTableModel extends FirewallTableModel {
	
	final static String[] COL_NAMES = { 
		"Interface",
		"From",
		"To",
		"Protocols",
		"Ports",
		"Destination",
		"Dest. Port"
	};
	
	/**
	 * Constructs a <code>NATTableModel</code> using the specified firewall.
	 * 
	 * @param	firewall	A DOM Document conforming to firewall.dtd
	 */
	public RedirectTableModel(Firewall firewall) {
		super(firewall, COL_NAMES);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		Redirect rdr = getFirewall().getRedirect(rowIndex);
		
		Object result = null;
		
		// Ease comparison
		String col = getColName(columnIndex);
		
		// Handle non-string data types
		if(col.equals("to")) {
			result = rdr.getTo();
		} else if(col.equals("from")) {
			result = rdr.getFrom();
		} else if(col.equals("interface")) {
			result = rdr.getInterface();
		} else if(col.equals("protocols")) {
			result = getVectorValue(rdr.enumerateProto());
		} else if(col.equals("ports")) {
			result = getVectorValue(rdr.enumeratePort());
		} else if(col.equals("destination")) {
			result = rdr.getDest();
		} else if(col.equals("dest. port")) {
			result = new Integer(rdr.getDestport());
		}

		// Missing XML elements return null.  Handle this appropriately.
		if(result == null) {
			return new String("");
		}

		return result;
	}
	
	/**
	 * Returns the DOM Tag name associated with a specified column
	 * 
	 * @param columnIndex	Column to retrieve the tag name for
	 * @return	DOM tag name of column
	 */
	private String getColName(int columnIndex) {
		return COL_NAMES[columnIndex].toLowerCase();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object newValue, int rowIndex, int columnIndex) {
		Redirect rdr = getFirewall().getRedirect(rowIndex);

		String col = getColName(columnIndex);

		if(col.equals("to")) {
			rdr.setTo(getStringValue(newValue));
		} else if(col.equals("from")) {
			rdr.setFrom(getStringValue(newValue));
		} else if(col.equals("interface")) {
			rdr.setInterface(getStringValue(newValue));
		} else if(col.equals("destination")) {
			rdr.setDest(getStringValue(newValue));
		} else if(col.equals("dest. port")) {
			rdr.setDestport(((Integer)newValue).intValue());
		} else if(col.equals("protocols")) {
			rdr.removeAllProto();
			
			Vector values = (Vector)newValue;
			
			for(int i = 0; i < values.size(); i++) {
				rdr.addProto((String)values.get(i));
			}
		} else if(col.equals("ports")) {
			rdr.removeAllPort();
			
			Vector values = (Vector)newValue;
			
			for(int i = 0; i < values.size(); i++) {
				rdr.addPort(getIntValue(values.get(i)));
			}
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}
	
	/**
	 * Create a new, empty NAT declaration.
	 * 
	 * @see #addBottom
	 */
	public void add() {
		Redirect rdr = new Redirect();
		getFirewall().addRedirect(rdr);
		
		fireTableRowsInserted(getRowCount(), getRowCount());
	}
	
	/**
	 * Removes the NAT declaration from the specified row in the table.
	 * 
	 * @param row	Row to remove.
	 */
	public void remove(int row) {
		getFirewall().removeRedirect(row);
		
		fireTableRowsDeleted(row, row);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return getFirewall().getRedirectCount();
	}

}
