/*
 * Copyright (c) 2004, Adam VanderHook <acidos@users.sf.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.bj.pfpro.gui.swing.table;

import java.util.Vector;

import net.bj.pfpro.firewall.Firewall;
import net.bj.pfpro.firewall.Rule;

/**
 * Handles showing of rules in a JTable.
 * 
 * @author acidos
 * @version $Id: RuleTableModel.java,v 1.5.4.2 2004/10/02 02:35:10 acidos Exp $
 * @see net.bj.pfpro.gui.swing.table.DOMTableModel
 */
public class RuleTableModel extends FirewallTableModel {
	
	final static String[] COL_NAMES = { 
		"Num",
		"Option",
		"Direction",
		"Quick",
		"Log",
		"Interface",
		"From",
		"To",
		"Protos",
		"Ports",
		"Users",
		"State",
		"Label",
		"Comment"
	};
	
	/**
	 * Default constructor
	 * 
	 * @param	firewall	The <code>Firewall</code> to use
	 */
	public RuleTableModel(Firewall firewall) {
		super(firewall, COL_NAMES);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		Rule rule = getFirewall().getRule(rowIndex);
		
		Object result = null;
		
		// Ease comparison
		String col = getColName(columnIndex);
		
		// Handle non-string data types
		if(col.equals("log")) {
			result = new Boolean(rule.getLog());
		} else if(col.equals("quick")) {
			result = new Boolean(rule.getQuick());
		} else if(col.equals("to")) {
			result = rule.getTo();
		} else if(col.equals("from")) {
			result = rule.getFrom();
		} else if(col.equals("num")) {
			result = new Integer(rule.getNum());
		} else if(col.equals("protos")) {
			result = getVectorValue(rule.enumerateProto());
		} else if(col.equals("users")) {
			result = getVectorValue(rule.enumerateUser());
		} else if(col.equals("ports")) {
			result = getVectorValue(rule.enumeratePort());
		} else if(col.equals("option")) {
			result = rule.getOption();
		} else if(col.equals("direction")) {
			result = rule.getDirection();
		} else if(col.equals("interface")) {
			result = rule.getInterface();
		} else if(col.equals("state")) {
			result = rule.getState();
		} else if(col.equals("comment")) {
			result = rule.getComment();
		} else if(col.equals("label")) {
			result = rule.getLabel();
		}

		// Missing XML elements return null.  Handle
		// this appropriately.
		if(result == null) {
			return new String("");
		}

		return result;
	}
	
	/**
	 * Returns the DOM Tag name associated with a specified column
	 * 
	 * @param columnIndex	Column to retrieve the tag name for
	 * @return	DOM tag name of column
	 */
	private String getColName(int columnIndex) {
		return COL_NAMES[columnIndex].toLowerCase();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object newValue, int rowIndex, int columnIndex) {
		Rule rule = getFirewall().getRule(rowIndex);

		String col = getColName(columnIndex);

		if(col.equals("log")) {
			rule.setLog(((Boolean)newValue).booleanValue());
		} else if(col.equals("quick")) {
			rule.setQuick(((Boolean)newValue).booleanValue());
		} else if(col.equals("to")) {
			rule.setTo(getStringValue(newValue));
		} else if(col.equals("from")) {
			rule.setFrom(getStringValue(newValue));
		} else if(col.equals("num")) {
			rule.setNum(((Integer)newValue).intValue());
		} else if(col.equals("option")) {
			rule.setOption((String)newValue);
		} else if(col.equals("direction")) {
			rule.setDirection(getStringValue(newValue));
		} else if(col.equals("interface")) {
			rule.setInterface(getStringValue(newValue));
		} else if(col.equals("state")) {
			rule.setState(getStringValue(newValue));
		} else if(col.equals("comment")) {
			rule.setComment(getStringValue(newValue));
		} else if(col.equals("label")) {
			rule.setLabel(getStringValue(newValue));
		} else if(col.equals("protos")) {
			rule.removeAllProto();
			
			Vector values = (Vector)newValue;
			
			for(int i = 0; i < values.size(); i++) {
				rule.addProto((String)values.get(i));
			}
		} else if(col.equals("users")) {
			rule.removeAllUser();
			
			Vector values = (Vector)newValue;
			
			for(int i = 0; i < values.size(); i++) {
				rule.addUser((String)values.get(i));
			}
		} else if(col.equals("ports")) {
			rule.removeAllPort();
			
			Vector values = (Vector)newValue;
			
			for(int i = 0; i < values.size(); i++) {
				rule.addPort(getIntValue(values.get(i)));
			}
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
// TODO make num column uneditable after implementing a sorting scheme for net.bj.pfpro.firewall.Rule
//
//		if(columnIndex == 0) {		// number ordering
//			return false;
//		} 
//		
		return super.isCellEditable(rowIndex, columnIndex);
	}
	
	/**
	 * Create a new, empty rule.  By default this is placed at the bottom.
	 * 
	 * @see #addBottom
	 */
	public void add() {
		addBottom();
	}
	
	/**
	 * Creates a new rule with a specific index
	 * 
	 * @param num	Index of the new rule
	 */
	private void add(int num) {
		Rule rule = new Rule();
		rule.setNum(num);
		getFirewall().addRule(rule);
		
		fireTableRowsInserted(getRowCount(), getRowCount());
	}

	/**
	 * Adds a new, blank rule to the bottom of the rule list.
	 */
	public void addBottom() {
		add(getFirewall().getRuleCount() + 1);
	}
	
	/**
	 * Adds a new, blank rule to the top of the rule list.
	 */
	public void addTop() {
		// TODO PLACE RULE PROPERLY
		add();
	}
	
	/**
	 * Adds a new, blank rule below the specified index
	 * 
	 * @param rowIndex	Index to precede the new rule
	 */
	public void addBelow(int rowIndex) {
		// TODO PLACE RULE PROPERLY
		add();
	}
	
	/**
	 * Adds a new blank rule above the current row.
	 * 
	 * @param rowIndex	The row to place the rule before
	 */
	public void addAbove(int rowIndex) {
		// TODO PLACE RULE PROPERLY
		add();
	}
	
	/**
	 * Removes the rule from the specified row in the table.
	 * 
	 * @param row	Row to remove.
	 */
	public void remove(int row) {
		getFirewall().removeRule(row);
		
		fireTableRowsDeleted(row, row);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return getFirewall().getRuleCount();
	}

}
